//---------------------------------------------------------------------------

#include "ValShdrFac.h"

//---------------------------------------------------------------------------
#include "ValShdrObj.h"
#include "ValShdr.h"

// ***** ClassFactory *****
  
ValShaderFactory::ValShaderFactory() : refCount(0) {}

// IUnknown methods of CheckerClassFactory                                                
STDMETHODIMP ValShaderFactory::QueryInterface(REFIID riid, LPVOID FAR* ppv)
{
   *ppv=NULL;

   if (IsEqualIID(riid, IID_IUnknown) || IsEqualIID(riid, IID_IClassFactory))
       *ppv=(LPVOID)this;

   if (*ppv!=NULL)
   {
       ((LPUNKNOWN)*ppv)->AddRef(); // Add reference if we give a pointer
       return NOERROR;
   }
   else
   {
       return ResultFromScode(E_NOINTERFACE);
   }
}


// Note that servers are not included in the local object count as per
// the OLE guidelines (Inside OLE, 2nd ed., p242)

STDMETHODIMP_(ULONG) ValShaderFactory::AddRef()
{
   return ++refCount;
}


STDMETHODIMP_(ULONG) ValShaderFactory::Release()
{
   if (--refCount == 0)
   {
       delete this; // No reference left, so delete the CheckerClassFactory
       return 0;
   }
   return refCount;
}

/*
 * ValShaderFactory::CreateInstance
 *
 * Parameters:
 *  pUnkOuter       LPUNKNOWN to the controlling IUnknown if we are
 *                  being used in an aggregation.
 *  riid            REFIID identifying the interface the caller
 *                  desires to have for the new object.
 *  ppvObj          LPVOID FAR* in which to store the desired
 *                  interface pointer for the new object.
 *
 * Return Value:
 *  HRESULT         NOERROR if successful, otherwise E_NOINTERFACE
 *                  if we cannot support the requested interface.
 */

STDMETHODIMP ValShaderFactory::CreateInstance(LPUNKNOWN pUnkOuter,
   REFIID riid, LPVOID FAR* ppvObj)
{
   EDPValShader *pObj = NULL;
   HRESULT hr;

   *ppvObj = NULL;
   hr = ResultFromScode(E_OUTOFMEMORY);

   // No support for aggregation
   if (pUnkOuter)
       return ResultFromScode(CLASS_E_NOAGGREGATION);

   //Create the object passing function to notify on destruction.
   if (IsEqualIID(riid, IID_IUnknown) || IsEqualIID(riid, IID_I3DExShader))
       pObj = new EDPValShader;
   else
       return ResultFromScode(E_NOINTERFACE);

   hr=pObj->QueryInterface(IID_I3DExShader, ppvObj);

   //Kill the object if initial creation failed.
   if (FAILED(hr))
       delete pObj;
   else
       ++global_count_Obj;

   return hr;
}


/*
 * CheckerClassFactory::LockServer
 *
 * Purpose:
 *  Increments or decrements the lock count of the DLL.  If the
 *  lock count goes to zero and there are no objects, the DLL
 *  is allowed to unload.  See DllCanUnloadNow in "Shadrdll.cpp".
 *
 * Parameters:
 *  fLock           BOOL specifying whether to increment or
 *                  decrement the lock count.
 *
 * Return Value:
 *  HRESULT         NOERROR always.
 */

STDMETHODIMP ValShaderFactory::LockServer(BOOL fLock)
{
   if (fLock)
       ++global_count_Lock;
   else
       --global_count_Lock;

   return NOERROR;
}


