//---------------------------------------------------------------------------

#include "ValShdrObj.h"

//---------------------------------------------------------------------------
#include "ValShdr.h"
#include "3DCoFail.h"      // Not sure exactly what this does, but examples include

// Define my GUIDS
/*
const GUID CLSID_EDPValShader = { 0xD64CBB10, 0x24CF,
   0x11D2, { 0x80, 0xDF, 0x20, 0xB6, 0xA4, 0x00, 0x00, 0x00 } };

const GUID IID_IEDPValShader = { 0x704749A0, 0x24DE,
   0x11D2, { 0x80, 0xDF, 0x20, 0xB6, 0xA4, 0x00, 0x00, 0x00 } };
*/

#undef INTERFACE
#define INTERFACE EDPValShader

EDPValShader::EDPValShader() : refCount(0)
{
   shaderPublicData.sliderValue = 50; // default value
}

EDPValShader::~EDPValShader()
{
   --global_count_Obj; // used by the DLL to know if it can be unloaded
}


// ****** IUnknown methods ******
STDMETHODIMP EDPValShader::QueryInterface(THIS_ REFIID riid, LPVOID FAR* ppvObj)
{
   *ppvObj=NULL;
  
   // The EDPValShader knows the interfaces of the parent Objects
   if (IsEqualIID(riid, IID_IUnknown))
       *ppvObj=(LPVOID)this;
   else if (IsEqualIID(riid, IID_I3DExShader))
       *ppvObj=(LPVOID)this;
   else if (IsEqualIID(riid, IID_I3DExDataExchanger))
       *ppvObj=(LPVOID)this;
   else if (IsEqualIID(riid, IID_I3DExtension))
       *ppvObj=(LPVOID)this;
   else if (IsEqualIID(riid, IID_IEDPValShader))
       *ppvObj=(LPVOID)this;

   // we must add reference if we return an interface
   if (*ppvObj != NULL)
   {
       ((LPUNKNOWN)*ppvObj)->AddRef();
       return NOERROR;
   }

   return ResultFromScode(E_NOINTERFACE);
}
                        
STDMETHODIMP_(ULONG) EDPValShader::AddRef(THIS)
{
   return ++refCount;
}
  
STDMETHODIMP_(ULONG) EDPValShader::Release(THIS)
{
   if (--refCount == 0)
   {
       delete this;
       return 0;
   }
   return refCount;
}

// ****** I3DExtension methods ******
STDMETHODIMP_(I3DExtension*) EDPValShader::Clone(THIS)
{
   EDPValShader* theClone = new EDPValShader;

   // Copy relevant data members
   theClone->shaderPublicData = shaderPublicData;

   theClone->AddRef();
   return theClone;
}

// Not sure exactly what next function does, but it was in example
STDMETHODIMP EDPValShader::ShellUtilitiesInit(THIS_ IShUtilities* shellUtilities)
{
   InitCoFailure(shellUtilities);
   return NOERROR;
}


// ****** I3DExDataExchanger methods ******
// Alternative to a PMAP for data exchange with the shell.
// NULL return indicates a PMAP is used.
STDMETHODIMP_(ExtensionDataMap*) EDPValShader::GetExtensionDataMap(THIS)
{
   return NULL;
}

// Give a pointer to the public data that the shell can modify
STDMETHODIMP_(void*) EDPValShader::GetExtensionDataBuffer(THIS)
{
   return ((void*) &(shaderPublicData));
}

// Called when the Shell modified the Properties Data.
// Can be used as trigger for preprocessing, but as this shader
// does no preprocessing, function simply returns.
STDMETHODIMP EDPValShader::ExtensionDataChanged(THIS)
{
   return NOERROR;
}

// Function is called when a button in the user interface (assuming
// the shader has such a button) is pushed. Is used to react to
// the user-initiated event. As this shader has no such buttons,
// function is not implmented.
STDMETHODIMP EDPValShader::HandleEvent(THIS_ ULONG sourceID)
{
   return ResultFromScode(E_NOTIMPL);
}

// The shell calls this function to get the ID for the various resources
// (e.g. COMP, PMAP, View) of the shader. In a Windows program, these
// are resources attached to the DLL. The source is in an '.rc' file.
STDMETHODIMP_(short) EDPValShader::GetResID(THIS)
{
  return 128;
}

// ****** I3DExShader methods ******
// Checks to see if two shaders are equal
STDMETHODIMP_(BOOLEAN) EDPValShader::IsEqualTo(THIS_ I3DExShader* aShader)
{
   EDPValShader *otherShader = NULL;
   BOOLEAN result = FALSE;

   // First, see if it's same type of shader
   aShader->QueryInterface(IID_IEDPValShader, & (LPVOID) otherShader);
   if (otherShader == NULL) return FALSE;

   // Second, see if state data is identical
   result = (otherShader->shaderPublicData.sliderValue ==
       shaderPublicData.sliderValue);

   otherShader->Release(); // Do not forget this
   return result;
}

// Determines if the shader will get the ShadingElem data structure on shading
// function calls. This data stucture seems to provide information on the extent
// of the shaded region on the object (likely to be relevant when shading paint
// shapes on an objects surface).
STDMETHODIMP_(BOOLEAN) EDPValShader::DependsOnAppliedExtent(THIS)
{
  return FALSE;
}

// Function should set the appropriate values in the ShadingFlags
// structure, which provides the shell with information on how the
// shader works and what parameters it needs during the shading function calls
STDMETHODIMP EDPValShader::GetShadingFlags(THIS_ ShadingFlags* theFlags)
{
  theFlags->fCallOnce = TRUE;      // Constant Shader: The shader needs to be called
                                   // only once for the whole object or shading area
  theFlags->fNeedsPoint = FALSE;   // Needs surface point in Global Coordi-nates
  theFlags->fNeedsNormal = FALSE;  // Needs surface Normal in Global Coordi-nates
  theFlags->fNeedsIsoU = FALSE;    // Needs U iso-parametric vector in Local Coordinates
  theFlags->fNeedsIsoV = FALSE;    // Needs V iso-parametric vector in Local Coordinates
  theFlags->fNeedsUV = FALSE;      // Needs u,v texture values
  theFlags->fNeedsPointLoc = FALSE;    // Needs surface point in Local Coordi-nates
  theFlags->fNeedsNormalLoc = FALSE;   // Needs surface Normal in Local Coordi-nates
  theFlags->fChangesNormal = FALSE;    // The shader changes the Normal. The new Normal
                                       // will have to be read in ShadingOut::fChangedNormal

  return NOERROR;
}

// Return value tells shell which method to use for the actual shading fuction calls
// All other methods should return E_NOTIMPL.
STDMETHODIMP_(ULONG) EDPValShader::GetPreferredOutput(THIS)
{
  return kUsesGetValue;
}

// A shading function method that handles all channels of a shader at once.
// Not implemented in this shader.
STDMETHODIMP EDPValShader::DoShade(THIS_ ShadingInOut* theShadingIO, ShadingElem* theShadingElem)
{
  return ResultFromScode(E_NOTIMPL);
}

// A shading function method that returns a value.
// In this shader, it is simply the value of the slider, scaled to the 0-1 range.
STDMETHODIMP EDPValShader::GetValue(THIS_ NUM3D* result, ShadingIn* theShadingIn,
   ShadingElem* theShadingElem)
{
   short sv = shaderPublicData.sliderValue;
   if (sv < 0) sv = 0;
   else if (sv > 100) sv = 100;

   *result = ((NUM3D) sv) / 100;

   return NOERROR;
}

// A shading function method that returns a color.
// Not implemented in this shader.
STDMETHODIMP EDPValShader::GetColor(THIS_ COLOR3D* result, ShadingIn* theShadingIn,
   ShadingElem* theShadingElem)
{
  return ResultFromScode(E_NOTIMPL);
}

// A shading function method that is reserved for future use.
// Not implemented in this shader.
STDMETHODIMP EDPValShader::GetVector(THIS_ VECTOR3D* result, ShadingIn* theShadingIn,
   ShadingElem* theShadingElem)
{
	return ResultFromScode(E_NOTIMPL);
}



